/**************************************************************************
*  Copyright (c) 2016 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  05.03.2016  mifi  First Version.
**************************************************************************/
#define __ETH_CONFIG_C__

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include <stdint.h>
#include <string.h>

#include "project.h"
#include "eth_phy_driver.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

#define PHY_TIMEOUT_MAX 255

/* PHY registers */
#define PHYIDR1         0x02
#define PHYIDR2         0x03

#define PHY_BCR         0                    /* Transceiver Basic Control Register */
#define PHY_Reset       ((uint16_t)0x8000)   /* PHY Reset */


/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static ETH_PHY_DRIVER PHYDriver;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  StartMIIWrite                                                        */
/*                                                                       */
/*  Starts a PHY write via the MII.                                      */
/*                                                                       */
/*  In    : Address, Reg, Value                                          */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void StartMIIWrite (uint8_t Address, uint16_t Reg, uint16_t Value)
{
   uint16_t Addr = ENET_MADR_PHYADDR(Address);

   /* Write value at PHY address and register */
   LPC_ETHERNET->MAC.MCMD = 0;
   LPC_ETHERNET->MAC.MADR = Addr | ENET_MADR_REGADDR(Reg);
   LPC_ETHERNET->MAC.MWTD = Value;
   
} /* StartMIIWrite */

/*************************************************************************/
/*  StartMIIRead                                                         */
/*                                                                       */
/*  Starts a PHY read via the MII.                                       */
/*                                                                       */
/*  In    : Address, Reg                                                 */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void StartMIIRead (uint8_t Address, uint16_t Reg)
{
   uint16_t Addr = ENET_MADR_PHYADDR(Address);

   /* Read value at PHY address and register */
   LPC_ETHERNET->MAC.MADR = Addr | ENET_MADR_REGADDR(Reg);
   LPC_ETHERNET->MAC.MCMD = ENET_MCMD_READ;
   
} /* StartMIIRead */

/*************************************************************************/
/*  PHYReadReg                                                           */
/*                                                                       */
/*  Read a register of the PHY.                                          */
/*                                                                       */
/*  In    : pPHY, Reg                                                    */
/*  Out   : none                                                         */
/*  Return: Value of the register                                        */
/*************************************************************************/
static uint16_t PHYReadReg (ETH_PHY_DRIVER *pPHY, uint16_t Reg)
{
   uint32_t dTimeout = 0;
   uint16_t wValue;

	/* Start register read */
   StartMIIRead(pPHY->Address, Reg);

   /* Check for the Busy flag */
   do
   {
      OSTimeWait(1);
      dTimeout++;
   
      if (false == Chip_ENET_IsMIIBusy(LPC_ETHERNET))
      {
         break;
      }
   } while (dTimeout < (uint32_t)PHY_TIMEOUT_MAX);

   /* Get value */
   wValue = Chip_ENET_ReadMIIData(LPC_ETHERNET);

   return(wValue);
} /* PHYReadReg */

/*************************************************************************/
/*  PHYWriteReg                                                          */
/*                                                                       */
/*  Write a register of the PHY.                                         */
/*                                                                       */
/*  In    : pPHY, Reg, Value                                             */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void PHYWriteReg (ETH_PHY_DRIVER *pPHY, uint16_t Reg, uint16_t Value)
{
   uint32_t dTimeout = 0;

   /* Start register write */
   StartMIIWrite(pPHY->Address, Reg, Value);
   
   /* Check for the Busy flag */
   do
   {
      OSTimeWait(1);
      dTimeout++;

      if (false == Chip_ENET_IsMIIBusy(LPC_ETHERNET))
      {
         break;
      }
   } while (dTimeout < (uint32_t)PHY_TIMEOUT_MAX);
   
} /* PHYWriteReg */

/*************************************************************************/
/*  FindPHY                                                              */
/*                                                                       */
/*  Find the PHY which is used.                                          */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
ETH_PHY_DRIVER *FindPHY (void)
{
   uint32_t Value;
   uint8_t  Addr;

   /*
    * Setup PHYDriver
    */
   memset(&PHYDriver, 0x00, sizeof(PHYDriver));    
   PHYDriver.ReadReg  = PHYReadReg;
   PHYDriver.WriteReg = PHYWriteReg;

   /*
    * Search the PHY...
    */
   for (Addr = 0; Addr < 32; Addr++)
   {    
      PHYDriver.Address = Addr; 
    
      /* Reset PHY */
      PHYWriteReg(&PHYDriver, PHY_BCR, PHY_Reset);
      
      /* Give PHY some time to reset */
      OSTimeWait(2);
    
      Value  = PHYReadReg(&PHYDriver, PHYIDR1);    
      Value  = Value << 16;
      Value |= PHYReadReg(&PHYDriver, PHYIDR2);
      Value &= PHY_ID_MASK;
   
      switch (Value)
      {
         case PHY_ID_DP83848C:
         {
            PHY_DP83848.ReadReg  = PHYReadReg;
            PHY_DP83848.WriteReg = PHYWriteReg;
            PHY_DP83848.Address  = Addr;
            return(&PHY_DP83848);
            break;   /*lint !e527*/
         }
         
         case PHY_ID_KSZ8721BL:
         {
            PHY_KSZ8721.ReadReg  = PHYReadReg;
            PHY_KSZ8721.WriteReg = PHYWriteReg;
            PHY_KSZ8721.Address  = Addr;
            return(&PHY_KSZ8721);
            break;   /*lint !e527*/
         }
      
         case PHY_ID_SMCS8720:
         {
            PHY_SMSC8720.ReadReg  = PHYReadReg;
            PHY_SMSC8720.WriteReg = PHYWriteReg;
            PHY_SMSC8720.Address  = Addr;
            return(&PHY_SMSC8720);
            break;   /*lint !e527*/
         }
      
         default:
         {
            /* Do noting */
            break;
         }
      }
   }      
      
   return(NULL);
} /* CheckPHY */

/*************************************************************************/
/*  GPIO_CONFIG                                                          */
/*                                                                       */
/*  Configures the GPIO ports.                                           */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void GPIO_CONFIG (void)
{
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0x1, 0,  (IOCON_FUNC1 | IOCON_MODE_INACT));   /*lint !e845*/
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0x1, 1,  (IOCON_FUNC1 | IOCON_MODE_INACT));   /*lint !e845*/
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0x1, 4,  (IOCON_FUNC1 | IOCON_MODE_INACT));   /*lint !e845*/
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0x1, 8,  (IOCON_FUNC1 | IOCON_MODE_INACT));   /*lint !e845*/
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0x1, 9,  (IOCON_FUNC1 | IOCON_MODE_INACT));   /*lint !e845*/
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0x1, 10, (IOCON_FUNC1 | IOCON_MODE_INACT));   /*lint !e845*/
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0x1, 14, (IOCON_FUNC1 | IOCON_MODE_INACT));   /*lint !e845*/
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0x1, 15, (IOCON_FUNC1 | IOCON_MODE_INACT));   /*lint !e845*/
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0x1, 16, (IOCON_FUNC1 | IOCON_MODE_INACT));   /*lint !e845*/
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0x1, 17, (IOCON_FUNC1 | IOCON_MODE_INACT));   /*lint !e845*/
//   Chip_IOCON_PinMuxSet(LPC_IOCON, 0x1, 27, (IOCON_FUNC4 | IOCON_MODE_INACT | IOCON_DIGMODE_EN)); /* CLKOUT */    /*lint !e845*/
} /* GPIO_CONFIG */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  BoardETHConfig                                                       */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: NULL / PHYDriver                                             */
/*************************************************************************/
ETH_PHY_DRIVER *BoardETHConfig (void)
{
   ETH_PHY_DRIVER *pPHY;
   
   /* 
    * Configure the GPIO ports for ethernet pins.
    * This is a macro depending of the board.
    */
   GPIO_CONFIG();

   /* 
    * Configure the Ethernet MAC/DMA 
    */
   Chip_ENET_Init(LPC_ETHERNET, true); /*lint !e747*/
   
   /* Setup MMI interface */
   Chip_ENET_SetupMII(LPC_ETHERNET, Chip_ENET_FindMIIDiv(LPC_ETHERNET, 2500000), 0);
   
   /*
    * Find the PHY which is used on the board.
    */
   pPHY = FindPHY();
   if (pPHY != NULL)
   {
      /* 
       * Configure Ethernet interrupt
       */
      NVIC_SetPriority(ETHERNET_IRQn, 3);
      NVIC_EnableIRQ(ETHERNET_IRQn);
   }

   return(pPHY);
} /* BoardETHConfig */

/*** EOF ***/
